#ifdef __cplusplus
extern "C"
{
#endif
#include "lib/healpix/healpix.h"
#ifdef __cplusplus
}
#endif

#include "grid_healpix.h"
#include <cstring>

HpIndexing
hp_get_indexing(const char *indexingName)
{
  HpIndexing indexing(HpIndexing::Undef);
  // clang-format off
  if      (strcmp(indexingName, "xy")   == 0) indexing = HpIndexing::XY;
  else if (strcmp(indexingName, "ring") == 0) indexing = HpIndexing::Ring;
  else if (strcmp(indexingName, "nest") == 0) indexing = HpIndexing::Nest;
  // clang-format on
  return indexing;
}

static int64_t
hp_xy_to_xy(int64_t ipix, int nside)
{
  (void)nside;
  return ipix;
}

void
hp_generate_coords(HpIndexing indexing, int nside, int64_t nvals, double *xvals, double *yvals, bool withBounds, double *xbounds, double *ybounds)
{
  int64_t (*convert_index)(int64_t ipix, int nside);
  // clang-format off
  convert_index = (indexing == HpIndexing::Ring) ? &healpixl_ring_to_xy :
                  (indexing == HpIndexing::Nest) ? &healpixl_nested_to_xy : &hp_xy_to_xy;
  // clang-format on

#ifdef _OPENMP
#pragma omp parallel for if (nvals > 99999) default(shared) schedule(static)
#endif
  for (int64_t ipix = 0; ipix < nvals; ipix++)
    {
      const auto index = convert_index(ipix, nside);

      healpixl_to_radec(index, nside, 0.5, 0.5, &xvals[ipix], &yvals[ipix]);

      if (withBounds)
        {
          healpixl_to_radec(index, nside, 0.0, 0.0, &xbounds[ipix * 4 + 0], &ybounds[ipix * 4 + 0]);
          healpixl_to_radec(index, nside, 1.0, 0.0, &xbounds[ipix * 4 + 1], &ybounds[ipix * 4 + 1]);
          healpixl_to_radec(index, nside, 1.0, 1.0, &xbounds[ipix * 4 + 2], &ybounds[ipix * 4 + 2]);
          healpixl_to_radec(index, nside, 0.0, 1.0, &xbounds[ipix * 4 + 3], &ybounds[ipix * 4 + 3]);
        }
    }
}
